from PyQt5.QtWidgets import QWidget, QVBoxLayout, QTreeWidget, \
    QTreeWidgetItem, QFileDialog, QPushButton, QHBoxLayout, \
    QPlainTextEdit, QCheckBox, QHeaderView, QMessageBox
from PyQt5.QtCore import Qt
import xml.etree.ElementTree as ET
from defusedxml.ElementTree import parse

# copied from: https://github.com/CaroAMN/Teamprojekt/blob/Protein_Analyzer/src/view/ConfigView.py
# in commit 516db62
# due to widget not being available in upstream master

class ConfigView(QWidget):
    def __init__(self, *args):
        QWidget.__init__(self, *args)

        self.path = ""

        self.tree = ET.ElementTree
        self.header = ['Name', 'Value', 'Type', 'Restrictions']
        self.NAMECOL = 0
        self.VALUECOL = 1
        self.TYPECOL = 2
        self.RESTRICTIONCOL = 3
        self.descriptions = {}
        self.drawTree = False

        self.treeWidget = QTreeWidget(self)
        self.treeWidget.setHeaderLabels(self.header)
        self.header = self.treeWidget.header()
        self.header.resizeSection(0, 150)
        self.header.resizeSection(1, 150)
        self.header.resizeSection(2, 150)
        # self.header.setMinimumSectionSize(50)

        self.treeWidget.itemSelectionChanged.connect(self.loadDescription)
        self.changeListener()

        btns = QWidget(self)
        loadbtn = QPushButton('Load')
        loadbtn.setMaximumWidth(80)
        savebtn = QPushButton('Save')
        savebtn.setMaximumWidth(80)
        loadbtn.clicked.connect(self.loadFile)
        savebtn.clicked.connect(self.saveFile)

        self.checkbox = QCheckBox('Show advanced parameters')
        self.checkbox.setChecked(True)
        self.checkbox.stateChanged.connect(self.drawTreeInit)

        self.textbox = QPlainTextEdit(self)
        self.textbox.setReadOnly(True)

        btnlayout = QHBoxLayout()
        layout = QVBoxLayout()

        btnlayout.addWidget(loadbtn)
        btnlayout.addWidget(savebtn)
        btnlayout.addWidget(self.checkbox)
        btns.setLayout(btnlayout)
        btns.setFixedWidth(500)
        layout.addWidget(self.treeWidget, 6)
        layout.addWidget(self.textbox, 1)
        layout.addWidget(btns, 0.5)

        self.setLayout(layout)
        self.resize(500, 720)

    def loadFile(self):
        file, _ = QFileDialog.getOpenFileName(
            self, "QFileDialog.getOpenFileName()", "",
            "All Files (*);;ini (*.ini)")

        self.path = file
        self.openXML(self.path)

    def openXML(self, path):
        """
        Loads a XML file with .ini tag, parses the xml into ET.ElementTree
        calls the drawTree function to draw a Tree with the loaded xml
        """

        """
        file, _ = QFileDialog.getOpenFileName(
            self, "QFileDialog.getOpenFileName()", "",
            "All Files (*);;ini (*.ini)")
        """

        file = path

        if file:
            self.tree = parse(file)
            self.root = self.tree.getroot()
            self.drawTreeInit()

            self.header.setSectionResizeMode(QHeaderView.ResizeToContents)

    def generateTreeWidgetItem(self, item: ET.Element) -> QTreeWidgetItem:
        """
        generates a QTreeWidgetItem with each column for an
        ET.Element (e.g. root)
        """
        treeitem = QTreeWidgetItem()
        treeitem.setFlags(treeitem.flags() | Qt.ItemIsEditable)
        try:
            treeitem.setText(0, item.attrib['name'])
        except KeyError:
            pass
        try:
            treeitem.setText(1, item.attrib['value'])
        except KeyError:
            pass
        try:
            treeitem.setText(2, item.attrib['type'])
        except KeyError:
            pass
        try:
            treeitem.setText(3, item.attrib['restrictions'])
        except KeyError:
            pass
        try:
            self.descriptions[item.attrib['name']] = item.attrib['description']
        except KeyError:
            pass

        return treeitem

    def drawTreeInit(self):
        """
        Initialises the treewidget, add the top level item
        and starts the main recursion
        """
        self.drawTreeActive = True
        self.treeWidget.clear()
        root = self.tree.getroot()
        for child in root:
            if self.checkbox.isChecked():
                childitem = self.generateTreeWidgetItem(child)
                self.treeWidget.addTopLevelItem(childitem)
                if len(child.getchildren()) > 0:
                    self.drawTreeRecursive(childitem, child)
            else:
                try:
                    if child.attrib['advanced'] == 'false':
                        childitem = self.generateTreeWidgetItem(child)
                        self.treeWidget.addTopLevelItem(childitem)
                        if len(child.getchildren()) > 0:
                            self.drawTreeRecursive(childitem, child)
                except KeyError:
                    childitem = self.generateTreeWidgetItem(child)
                    self.treeWidget.addTopLevelItem(childitem)
                    if len(child.getchildren()) > 0:
                        self.drawTreeRecursive(childitem, child)
        self.treeWidget.expandAll()
        self.drawTreeActive = False

    def drawTreeRecursive(self, nodeitem: QTreeWidgetItem, node: ET.Element):
        """
        Draws a tree for the loaded XML file
        The checkbox "show advanced options" is implemented here as well
        it will only draw those items, which have the according advanced flag
        """
        for subnode in node:
            if self.checkbox.isChecked():
                subitem = self.generateTreeWidgetItem(subnode)
                nodeitem.addChild(subitem)
                if len(subnode.getchildren()) > 0:
                    self.drawTreeRecursive(subitem, subnode)
            else:
                try:
                    if subnode.attrib['advanced'] == 'false':
                        subitem = self.generateTreeWidgetItem(subnode)
                        nodeitem.addChild(subitem)
                        if len(subnode.getchildren()) > 0:
                            self.drawTreeRecursive(subitem, subnode)
                except KeyError:
                    subitem = self.generateTreeWidgetItem(subnode)
                    nodeitem.addChild(subitem)
                    if len(subnode.getchildren()) > 0:
                        self.drawTreeRecursive(subitem, subnode)

    def loadDescription(self):
        """
        Shows the description of the configuration parameter in the textbox
        """
        getSelected = self.treeWidget.selectedItems()
        if getSelected:
            try:
                node = getSelected[0].text(0)
                self.textbox.setPlainText(self.descriptions[node])
            except KeyError:
                node = getSelected[0].parent().text(0)
                self.textbox.setPlainText(self.descriptions[node])

    def changeListener(self):
        self.treeWidget.itemChanged.connect(self.editField)

    def editField(self):
        if not self.drawTreeActive:
            itemchanged = self.treeWidget.currentItem()
            itemparent = itemchanged.parent()
            itemname = itemchanged.text(self.NAMECOL)
            parentname = itemparent.text(self.NAMECOL)
            newvalue = itemchanged.text(self.VALUECOL)
            restrictions = itemchanged.text(self.RESTRICTIONCOL)
            types = itemchanged.text(self.TYPECOL)

            reschecked = self.checkRestrictionString(newvalue, restrictions)
            typechecked = self.checkTypeRestrictions(newvalue, types)

            if reschecked and typechecked:
                for parent in self.tree.iter('NODE'):
                    if parent.attrib['name'] == parentname:
                        for child in parent:
                            if child.attrib['name'] == itemname:
                                child.attrib['value'] = newvalue
            elif typechecked:
                QMessageBox.about(self, "Warning", "Please only, " +
                                  "modify according to Restrictions")
            else:
                QMessageBox.about(self, "Warning", "Please only, " +
                                  "modify according to Typerestrictions")

            self.drawTreeInit()

    def checkRestrictionString(self,
                               newvalue: type, restrictions: str) -> bool:
        if restrictions != "":
            if newvalue not in restrictions:
                reschecked = False
            else:
                reschecked = True
        else:
            reschecked = True

        return reschecked

    def checkTypeRestrictions(self, newvalue: type, types: str) -> bool:
        if types != "":
            try:
                float(newvalue)
                if len(newvalue.split('.')) == 2:
                    valtype = "double"
                else:
                    valtype = "int"
            except ValueError:
                valtype = "string"
            if valtype not in types:
                typechecked = False
            else:
                typechecked = True
        else:
            typechecked = True

        return typechecked

    def saveTmpFile(self):
        file = "tmp.ini"
        if file:
            self.tree.write(file)

    def saveFile(self):
        file, _ = QFileDialog.getSaveFileName(
            self, "QFileDialog.getSaveFileName()", "",
            "All Files (*);;ini (*.ini)")

        if file:
            self.tree.write(file)



from collections import namedtuple

import numpy as np
import pyqtgraph as pg
from PyQt5.QtCore import Qt
from pyqtgraph import PlotWidget

# structure for annotation (here for reference)
PeakAnnoStruct = namedtuple(
    "PeakAnnoStruct",
    "mz intensity text_label \
                            symbol symbol_color",
)
LadderAnnoStruct = namedtuple(
    "LadderAnnoStruct",
    "mz_list \
                            text_label_list color",
)

pg.setConfigOption("background", "w")  # white background
pg.setConfigOption("foreground", "k")  # black peaks


class SpectrumWidget(PlotWidget):
    def __init__(self, parent=None, dpi=100):
        PlotWidget.__init__(self)
        self.setLimits(yMin=0, xMin=0)
        self.setMouseEnabled(y=False)
        self.setLabel("bottom", "m/z")
        self.setLabel("left", "intensity")
        self.highlighted_peak_label = None
        self.peak_annotations = None
        self.ladder_annotations = None
        # numpy arrays for fast look-up
        self._mzs = np.array([])
        self._ints = np.array([])
        self.getViewBox().sigXRangeChanged.connect(self._autoscaleYAxis)
        self.getViewBox().sigRangeChangedManually.connect(
            self.redrawLadderAnnotations
        )  # redraw anno
        self.proxy = pg.SignalProxy(
            self.scene().sigMouseMoved, rateLimit=60, slot=self._onMouseMoved
        )

    def setSpectrum(
            self, spectrum, zoomToFullRange=False
    ):  # add a default value for displaying all peaks
        self.plot(clear=True)
        self.zoomToFullRange = zoomToFullRange  # relevant in redrawPlot()
        # delete old highlighte "hover" peak
        """se_comment: changed != to is not"""
        if self.highlighted_peak_label is not None:
            self.removeItem(self.highlighted_peak_label)
            self.highlighted_peak_label = None
        self.spec = spectrum
        self._mzs, self._ints = self.spec.get_peaks()
        self._autoscaleYAxis()
        # for annotation in ControllerWidget
        self.minMZ = np.amin(self._mzs)
        self.maxMZ = np.amax(self._mzs)
        self.redrawPlot()

    def setPeakAnnotations(self, p_annotations):
        self.peak_annotation_list = p_annotations

    def setLadderAnnotations(self, ladder_visible=[]):
        self._ladder_visible = ladder_visible  # LadderAnnoStruct

    def clearLadderAnnotation(self, ladder_key_to_clear):
        try:
            if ladder_key_to_clear in self._ladder_anno_lines.keys():
                self._clear_ladder_item(ladder_key_to_clear)
        except (AttributeError, NameError):
            return

    def redrawPlot(self):
        self.plot(clear=True)
        if self.zoomToFullRange:
            self.setXRange(self.minMZ, self.maxMZ)
        self._plot_spectrum()
        self._clear_annotations()
        self._plot_peak_annotations()
        self._plot_ladder_annotations()

    def redrawLadderAnnotations(self):
        self._plot_ladder_annotations()

    def _autoscaleYAxis(self):
        x_range = self.getAxis("bottom").range
        if x_range == [
            0,
            1,
        ]:  # workaround for axis sometimes not being set
            # TODO: check if this is resovled
            x_range = [np.amin(self._mzs), np.amax(self._mzs)]
        self.currMaxY = self._getMaxIntensityInRange(x_range)
        if self.currMaxY:
            self.setYRange(0, self.currMaxY, update=False)

    def _plot_peak_annotations(self):
        try:
            self.peak_annotation_list
        except (AttributeError, NameError):
            return

        if self.peak_annotation_list is not None:
            for item in self.peak_annotation_list:  # item : PeakAnnoStruct
                self.plot(
                    [item.mz],
                    [item.intensity],
                    symbol=item.symbol,
                    symbolBrush=pg.mkBrush(item.symbol_color),
                    symbolSize=14,
                )
                if item.text_label:
                    label = pg.TextItem(
                        text=item.text_label, color=item.symbol_color, anchor=(
                            0.5, 1)
                    )
                    self.addItem(label)
                    label.setPos(item.mz, item.intensity)

    def _getMaxIntensityInRange(self, xrange):
        left = np.searchsorted(self._mzs, xrange[0], side="left")
        right = np.searchsorted(self._mzs, xrange[1], side="right")
        return np.amax(self._ints[left:right], initial=1)

    def _plot_spectrum(self):
        bargraph = pg.BarGraphItem(x=self._mzs, height=self._ints, width=0)
        self.addItem(bargraph)

    def _plot_ladder_annotations(self):
        try:
            self._ladder_visible
        except (AttributeError, NameError):
            return
        try:
            self.currMaxY
        except (AttributeError, NameError):
            self.currMaxY = self._getMaxIntensityInRange(
                self.getAxis("bottom").range)

        xlimit = [self._mzs[0], self._mzs[-1]]
        for ladder_key, lastruct in self._ladder_visible.items():
            if ladder_key in self._ladder_anno_lines.keys():  # update
                self._ladder_anno_lines[ladder_key][0].setData(
                    [xlimit[0], xlimit[1]], [self.currMaxY, self.currMaxY]
                )  # horizontal line
                cntr = 0
                for x in lastruct.mz_list:
                    self._ladder_anno_lines[ladder_key][cntr + 1].setData(
                        [x, x], [0, self.currMaxY]
                    )
                    self._ladder_anno_labels[ladder_key][cntr].setPos(
                        x, self.currMaxY
                    )  # horizon line doesn't have label
                    cntr += 1
            else:  # plot
                pen = pg.mkPen(lastruct.color, width=2, style=Qt.DotLine)
                self._ladder_anno_lines[ladder_key] = []
                self._ladder_anno_labels[ladder_key] = []

                self._ladder_anno_lines[ladder_key].append(
                    # horizon line. index 0
                    self.plot(
                        [xlimit[0], xlimit[1]], [
                            self.currMaxY, self.currMaxY], pen=pen
                    )
                )
                """se_comment: hard-refactor to comply to pep8"""
                z = zip(lastruct.mz_list, lastruct.text_label_list)
                for x, txt_label in z:
                    self._ladder_anno_lines[ladder_key].append(
                        self.plot([x, x], [0, self.currMaxY], pen=pen)
                    )
                    label = pg.TextItem(
                        text=txt_label, color=lastruct.color, anchor=(1, -1)
                    )
                    label.setPos(x, self.currMaxY)
                    label.setParentItem(
                        self._ladder_anno_lines[ladder_key][-1])
                    self._ladder_anno_labels[ladder_key].append(label)

    def _clear_annotations(self):
        self._ladder_visible = dict()
        self._ladder_anno_lines = dict()
        self._ladder_anno_labels = dict()

    def _clear_peak_annotations(self):
        self.peak_annotation_list = None

    def _clear_ladder_item(self, key):
        for anno in self._ladder_anno_lines[key]:
            anno.clear()
        for pos in self._ladder_anno_labels[key]:
            pos.setPos(0, 0)
        del self._ladder_anno_lines[key]
        del self._ladder_anno_labels[key]

    def _onMouseMoved(self, evt):
        pos = evt[0]  # using signal proxy
        # turns original arguments into a tuple
        if self.sceneBoundingRect().contains(pos):
            mouse_point = self.getViewBox().mapSceneToView(pos)
            pixel_width = self.getViewBox().viewPixelSize()[0]
            left = np.searchsorted(
                self._mzs, mouse_point.x() - 4.0 * pixel_width, side="left"
            )
            right = np.searchsorted(
                self._mzs, mouse_point.x() + 4.0 * pixel_width, side="right"
            )
            if left == right:  # none found -> remove text
                """se_comment: changed != to is not"""
                if self.highlighted_peak_label is not None:
                    self.highlighted_peak_label.setText("")
                return
            # get point in range with minimum squared distance
            dx = np.square(np.subtract(self._mzs[left:right], mouse_point.x()))
            dy = np.square(np.subtract(
                self._ints[left:right], mouse_point.y()))
            idx_max_int_in_range = np.argmin(np.add(dx, dy))
            x = self._mzs[left + idx_max_int_in_range]
            y = self._ints[left + idx_max_int_in_range]
            """se_comment: changed == to is"""
            if self.highlighted_peak_label is None:
                self.highlighted_peak_label = pg.TextItem(
                    text="{0:.3f}".format(x),
                    color=(100, 100, 100),
                    anchor=(0.5, 1.5)
                )
                self.addItem(
                    self.highlighted_peak_label, ignoreBounds=True
                )  # ignore bounds to prevent rescaling of axis
                # if the text item touches the border
            self.highlighted_peak_label.setText("{0:.3f}".format(x))
            self.highlighted_peak_label.setPos(x, y)
        else:
            # mouse moved out of visible area: remove highlighting item
            """se_comment: changed != to is not"""
            if self.highlighted_peak_label is not None:
                self.highlighted_peak_label.setText("")



import sys

import pyqtgraph as pg
from PyQt5.QtWidgets import QApplication
from PyQt5.QtWidgets import QMainWindow
from PyQt5.QtWidgets import QVBoxLayout
from PyQt5.QtWidgets import QWidget
from PyQt5.QtWidgets import QAction

pg.setConfigOption("background", "w")  # white background
pg.setConfigOption("foreground", "k")  # black peaks


class GUI_EXAMPLE_BASE(QMainWindow):
    def __init__(self):
        QMainWindow.__init__(self)
        self.resize(800, 601)
        self._initUI()

    def _initUI(self):
        self.setWindowTitle("Example Widget Viewer")
        self.centerWidget = QWidget(self)
        self.setCentralWidget(self.centerWidget)
        self.layout = QVBoxLayout(self.centerWidget)

        self._setMainMenu()
        self._setExitButton()

    def _setMainMenu(self):
        mainMenu = self.menuBar()
        mainMenu.setNativeMenuBar(False)
        self.titleMenu = mainMenu.addMenu("PyOpenMS")

    def _setExitButton(self):
        exitButton = QAction("Exit", self)
        exitButton.setShortcut("Ctrl+Q")
        exitButton.setStatusTip("Exit application")
        exitButton.triggered.connect(self.close)
        self.titleMenu.addAction(exitButton)

    def closeEvent(self, event):
        event.accept()

    def setExampleWidget(self, widget):
        self.widget = widget
        self.layout.addWidget(self.widget)


if __name__ == "__main__":
    app = QApplication(sys.argv)
    ex = GUI_EXAMPLE_BASE()
    ex.show()
    sys.exit(app.exec_())



import os
import sys
from collections import namedtuple

import pyqtgraph as pg
from PyQt5.QtWidgets import (
    QApplication,
    QMainWindow,
    QVBoxLayout,
    QWidget,
    QDesktopWidget,
    QAction,
    QFileDialog,
)

sys.path.insert(0, "../view")
from ControllerWidget import ControllerWidget

# structure for annotation (here for reference)
PeakAnnoStruct = namedtuple(
    "PeakAnnoStruct",
    "mz intensity text_label \
                            symbol symbol_color",
)
LadderAnnoStruct = namedtuple(
    "LadderAnnoStruct",
    "mz_list \
                            text_label_list color",
)

pg.setConfigOption("background", "w")  # white background
pg.setConfigOption("foreground", "k")  # black peaks


class App(QMainWindow):
    def __init__(self):
        QMainWindow.__init__(self)
        self.resize(1000, 1000)  # window size
        self.initUI()

    def initUI(self):
        self.setWindowTitle("pyOpenMS ID Viewer")
        self.center()

        # layout
        self.setMainMenu()
        self.centerWidget = QWidget(self)
        self.setCentralWidget(self.centerWidget)
        self.windowLay = QVBoxLayout(self.centerWidget)

        # default widget
        self.setWidgets()

    def setWidgets(self):
        if self.windowLay.count() > 0:
            self.clearLayout(self.windowLay)
        self.widgets = ControllerWidget(self)
        self.windowLay.addWidget(self.widgets)

    def setMainMenu(self):
        mainMenu = self.menuBar()
        mainMenu.setNativeMenuBar(False)

        self.titleMenu = mainMenu.addMenu("PyOpenMS")
        self.fileMenu = mainMenu.addMenu("File")
        # helpMenu = mainMenu.addMenu('Help')
        self.toolMenu = mainMenu.addMenu("Tools")

        self.setTitleMenu()
        self.setFileMenu()
        self.setToolMenu()

    def setTitleMenu(self):
        self.setExitButton()

    def setFileMenu(self):
        # open mzml file
        mzmlOpenAct = QAction("Open file", self)
        mzmlOpenAct.setShortcut("Ctrl+O")
        mzmlOpenAct.setStatusTip("Open new file")
        mzmlOpenAct.triggered.connect(self.openFileDialog)
        self.fileMenu.addAction(mzmlOpenAct)

    def setToolMenu(self):
        # for overriding
        return

    def clearLayout(self, layout):
        for i in reversed(range(layout.count())):
            layout.itemAt(i).widget().setParent(None)

    def openFileDialog(self):
        fileName, _ = QFileDialog.getOpenFileName(
            self, "Open File ", "", "idXML Files (*.idXML)"
        )
        if fileName:
            print("opening...", fileName)
            self.setWidgets()
            self.widgets.loadFileMzML(os.path.splitext(fileName)[0] + ".mzML")
            self.widgets.loadFileIdXML(fileName)

    def center(self):
        qr = self.frameGeometry()
        cp = QDesktopWidget().availableGeometry().center()
        qr.moveCenter(cp)
        self.move(qr.topLeft())

    def setExitButton(self):
        exitButton = QAction("Exit", self)
        exitButton.setShortcut("Ctrl+Q")
        exitButton.setStatusTip("Exit application")
        exitButton.triggered.connect(self.close)
        self.titleMenu.addAction(exitButton)

    def closeEvent(self, event):
        event.accept()


if __name__ == "__main__":
    app = QApplication(sys.argv)
    ex = App()
    ex.show()
    sys.exit(app.exec_())



import sys

import pyopenms
from GUI_EXAMPLE_BASE import GUI_EXAMPLE_BASE
from PyQt5.QtCore import QObject
from PyQt5.QtWidgets import QApplication

sys.path.insert(0, "../view")
from TICWidget import TICWidget


# mock object to test the mouse click signal by TICWidget


class TestMouseClick(QObject):
    old_variable = (0, 0)

    def __init__(self):
        QObject.__init__(self)

    def printRT(self, rt):
        print("Mouse clicked at RT=" + str(rt))

    def printRTBounds(self, start_rt, stop_rt):
        if TestMouseClick.old_variable != (start_rt, stop_rt):
            print("RT Bounds: ", start_rt, stop_rt)

        TestMouseClick.old_variable = (start_rt, stop_rt)


if __name__ == "__main__":
    app = QApplication(sys.argv)
    ex = GUI_EXAMPLE_BASE()  # plain QMainWindow with basic layout and menu bar

    # load spectra and add example widget to window
    exp = pyopenms.MSExperiment()
    pyopenms.MzMLFile().load("../data/190509_Ova_native_25ngul_R.mzML", exp)
    example_widget = TICWidget(ex)
    example_widget.setTIC(exp.getTIC())
    mouse_click_test = TestMouseClick()
    example_widget.sigRTClicked.connect(mouse_click_test.printRT)
    example_widget.sigSeleRTRegionChangeFinished.connect(
        mouse_click_test.printRTBounds)
    ex.setExampleWidget(example_widget)
    ex.show()
    sys.exit(app.exec_())



import sys


# Example for mass decomposition (mass explanation)
# Internal residue masses
# (as observed e.g. as mass shifts in tandem mass spectra)
# are decomposed in possible amino acid strings that match in mass.
from pyopenms.pyopenms_1 import Residue
from pyopenms.pyopenms_6 import MassDecompositionAlgorithm
from pyopenms.pyopenms_7 import ResidueDB

mass = float(sys.argv[1])  # provide mass as first command line parameter
residues = ResidueDB().getResidues(b"Natural19WithoutI")

print("Trying to decompose " + str(mass) + " into sums of residues:")
for r in residues:
    print(
        r.getOneLetterCode().decode() +
        "\t" +
        str(r.getMonoWeight(Residue.ResidueType.Internal))
    )

print("Mass explanations by fast algorithm:")
# fast algorithm based on integer mass decomposition
# returns a unique set of compositions
# (e.g.: only one of 'TSG' and 'GST' is reported)
md_alg = MassDecompositionAlgorithm()
param = md_alg.getParameters()
param.setValue("tolerance", 0.05)
param.setValue("residue_set", b"Natural19WithoutI")
md_alg.setParameters(param)
decomps = []
md_alg.getDecompositions(decomps, mass)
for d in decomps:
    print(d.toExpandedString().decode())

# for comparision:
# slow/naive algorithm to generate all decompositions
# (use only very small masses e.g. < 300 Da)
# def recurse(mass_sum, peptide):
#  if abs(mass_sum - mass) < 0.05:
#    print(peptide + "\t" + str(mass_sum))
#  for r in residues:
#    new_mass = mass_sum + r.getMonoWeight(Residue.ResidueType.Internal)
#    if new_mass < mass + 0.05:
#      recurse(new_mass, peptide# + r.getOneLetterCode().decode())
#
# print("Mass explanations by naive algorithm:")
# recurse(0, "")



  
import sys

from PyQt5.QtWidgets import QHBoxLayout, QPushButton, QVBoxLayout, QWidget, QSplitter, QMainWindow, QLabel, QLineEdit, QApplication, QAction, QMessageBox
from PyQt5.QtCore import Qt
from PyQt5.QtGui import QFont

class HomeTabWidget(QMainWindow):
    """class to create HomeTabWidget which contains general data about using GUI_Tabs
        ...
        Methods
        -------
        __init__(self)
            defines window size and title

        HomeWindow(self)
            creates window, contains textwidget and textboxes to manually change parameters

        check_parameters(self)
            checks whether parameters have been changed and clears textboxes
    """
    
    def __init__(self):
        """ defines window size and title and sets default parameter values
        ...
        Attributes
        ----------
        title : string
            title of widget

        left, top, width, height : int
            attributes regarding size of widget

        loadedThreads : string
            'threads' parameter of command, default value is 1

        loadedFDR : string
            'proteinFDR' parameter of command, default value is 0.3
        """
        super().__init__()
        self.title = 'Home'
        self.left = 100
        self.top = 100
        self.width = 600
        self.height = 500
        self.setWindowTitle(self.title)
        self.setGeometry(self.left, self.top, self.width, self.height)
        self.setStyleSheet("background-color:lightgrey;");
        self.HomeWindow()
        
        self.loadedThreads = "1"
        self.loadedFDR = "0.3"
    def HomeWindow(self):
        
        """
        Used to briefly introduce functionality of GUI_Tabs, allows to manually set parameters for cmd command
        ...
        Attributes
        ----------
        textwidget : QWidget
            main widget that contains text

        vertical : QVBoxLayout
            main layout of textwidget
        
        HomeTitle : QLabel
            label that contains headline of text

        Introduction : QLabel
            label that contains introduction text

        EditParameters : QLabel
            label that contains information to edit parameters

        Proteomics : QLabel
            label that contains text about running ProteomicsLFQ command

        ThreadText : QLabel
            label for 'Threads' textbox

        FDRText : QLabel
            label for 'FDR' textbox

        horizontal1, horizontal2, horizontal3 : QHBoxLayout
             horizontal layouts for different levels

        saveButton : QPushButton
            button to save changed parameters
        
        """
        self.textwidget = QWidget(self)
        self.vertical = QVBoxLayout(self.textwidget)
        #set margins
        self.vertical.setContentsMargins(5, 15, 100, 15)

        #set labels
        self.HomeTitle = QLabel()
        self.Introduction = QLabel()
        self.EditParameters = QLabel()
        self.Proteomics = QLabel()

        self.HomeTitle.setText("<font color = 'darkblue'>Welcome!</font>")
        self.HomeTitle.setFont(QFont("Arial",16))
        self.HomeTitle.move(100,50)
        IntroductionText = """
            This application can be used to display experimental data.
            
            You can choose to manually enter files in each tab or select a folder that contains all data.
            
            For the latter method, simply click 'Load Data'-Button above.
            """
        
        self.Introduction.setText(IntroductionText)
        self.Introduction.setFont(QFont("Arial",12))

        EditText = """
            If you want to edit the parameters 'Threads' or 'proteinFDR', you can do it here:
            """
        self.EditParameters.setText(EditText)
        self.EditParameters.setFont(QFont("Arial",12))

        #set text for textboxes
        self.ThreadText = QLabel()
        self.ThreadText.setText("   Threads:")
        self.ThreadText.setFont(QFont("Arial",12))
        self.FDRText = QLabel()
        self.FDRText.setText("  proteinFDR:")
        self.FDRText.setFont(QFont("Arial",12))

        self.horizontal1 = QHBoxLayout()
        self.horizontal2 = QHBoxLayout()
        self.horizontal3 = QHBoxLayout()
        
        #set Textboxes
        self.ThreadTextBox = QLineEdit(self)
        self.FDRTextBox = QLineEdit(self)
        self.ThreadTextBox.setFixedSize(210,30)
        self.FDRTextBox.setFixedSize(210,30)

        #set button to save parameters
        self.saveButton = QPushButton('Save Parameters!', self)
        self.saveButton.setFont(QFont("Arial",11))
        self.saveButton.setFixedSize(130,40)
        self.saveButton.clicked.connect(self.check_parameters)
        
        ProteomicsText = """
            If you're done, you can execute ProteomicsLFQ by clicking the 'Run ProteomicsLFQ'-Button!
            """
        self.Proteomics.setText(ProteomicsText)
        self.Proteomics.setFont(QFont("Arial",12))

        #add widgets to layouts
        self.textwidget.setLayout(self.vertical)
        
        self.vertical.addWidget(self.HomeTitle)
        self.vertical.addWidget(self.Introduction)
        self.vertical.addWidget(self.EditParameters)
        self.vertical.addLayout(self.horizontal3)
        self.horizontal3.addLayout(self.horizontal1)
        self.horizontal3.addLayout(self.horizontal2)
        
        self.horizontal1.addStretch(4)
        self.horizontal2.addStretch(4)
        self.horizontal3.addStretch(0)
        
        self.horizontal1.addWidget(self.ThreadText)
        self.horizontal1.addWidget(self.ThreadTextBox)

        self.horizontal2.addWidget(self.FDRText)
        self.horizontal2.addWidget(self.FDRTextBox)
        self.horizontal3.addWidget(self.saveButton)

        self.vertical.addWidget(self.Proteomics)

        self.setCentralWidget(self.textwidget)
        self.show()
        
    def check_parameters(self):
        """ checks whether parameters have been changed and clears textboxes"""
        if self.ThreadTextBox.text() == "":
            self.loadedThreads = "1"
        else: self.loadedThreads = self.ThreadTextBox.text()
         
        if self.FDRTextBox.text() == "":
            self.loadedFDR = "0.3"
        else: self.loadedFDR = self.FDRTextBox.text()
        self.ThreadTextBox.clear()
        self.FDRTextBox.clear()
         
        
"""
def main():
        app = QApplication(sys.argv)
        ex = HomeTabWidget()
        sys.exit(app.exec_())
if __name__ == '__main__':
        main()
        
 """    



from PyQt5.QtCore import Qt, QPointF
from PyQt5.QtGui import (
    QFont,
    QFontMetricsF,
    QPainter,
    QColor,
    QPen,
    QBrush,
    QSpacerItem,
    QSizePolicy,
)
from PyQt5.QtWidgets import QWidget, QHBoxLayout


class SequenceIonsWidget(QWidget):
    """
    Used for creates a window for a peptide sequence with its given ions,
    which is adjusted to the sequence size.
    To avoid contortions of the window, spaceritems are added.

    """

    HEIGHT = 0
    WIDTH = 0
    SUFFIX_HEIGHT = 0

    def __init__(self, *args):
        QWidget.__init__(self, *args)

        self.initUI()

    def initUI(self):
        self.mainlayout = QHBoxLayout(self)
        self.mainlayout.setContentsMargins(0, 0, 0, 0)
        self.container = QWidget(self)
        self.container.setStyleSheet("background-color:white;")

        self.setWindowTitle("SequenceIons Viewer")
        self.seqIons_layout = QHBoxLayout(self.container)

        # change default setting of 11 px
        self.seqIons_layout.setContentsMargins(0, 0, 0, 0)
        self._pep = observed_peptide()
        # resize window to fit peptide size
        self.resize()

        self._pep.setSizePolicy(QSizePolicy.Minimum, QSizePolicy.Fixed)
        self._pep.setMinimumSize(
            SequenceIonsWidget.WIDTH, SequenceIonsWidget.HEIGHT)
        self.seqIons_layout.addItem(
            QSpacerItem(
                40,
                SequenceIonsWidget.HEIGHT,
                QSizePolicy.MinimumExpanding,
                QSizePolicy.Minimum,
            )
        )
        self.seqIons_layout.addWidget(self._pep)
        self.seqIons_layout.addItem(
            QSpacerItem(
                40,
                SequenceIonsWidget.HEIGHT,
                QSizePolicy.MinimumExpanding,
                QSizePolicy.Minimum,
            )
        )

        self.setFixedHeight(SequenceIonsWidget.HEIGHT)
        self.mainlayout.addWidget(self.container)
        self.show()

    def resize(self):
        """
        The integer 8 represents the additional space needed
        for the in addition drawn lines. The 18 represents the
        monospace width.

        """
        if len(self._pep.sequence) == 0:
            SequenceIonsWidget.WIDTH = 0
        else:
            SequenceIonsWidget.WIDTH = \
                (len(self._pep.sequence) * 18) + \
                (len(self._pep.sequence) - 1) * 8
        self.calculateHeight()

    def calculateHeight(self):
        """
        Calculate window height adjusting to sequence height
        with possible ions and set default setting in case of no
        ions.

        """
        prefix = self._pep.prefix
        suffix = self._pep.suffix

        if suffix == {}:
            max_ion_suff = 1
        else:
            max_ion_suff = len(
                suffix[max(suffix, key=lambda key: len(suffix[key]))])

        if prefix == {}:
            max_ion_pre = 1
        else:
            max_ion_pre = len(
                prefix[max(prefix, key=lambda key: len(prefix[key]))])

        metrics_pep = QFontMetricsF(self._pep.getFont_Pep())
        height_pep = metrics_pep.height()

        metrics_ion = QFontMetricsF(self._pep.getFont_Ion())
        height_ion = metrics_ion.height()

        # window height calculated with the sum of max prefix and suffix height
        height_ion_pre = height_ion * max_ion_pre + 15
        SequenceIonsWidget.SUFFIX_HEIGHT = height_ion * max_ion_suff + 5
        SequenceIonsWidget.HEIGHT = \
            (
                height_pep + height_ion_pre + SequenceIonsWidget.SUFFIX_HEIGHT
            )

    def setPeptide(self, seq):
        self._pep.setSequence(seq)
        self.updateWindow()

    def setSuffix(self, suff):
        self._pep.setSuffix(suff)
        self.updateWindow()

    def setPrefix(self, pre):
        self._pep.setPrefix(pre)
        self.updateWindow()

    def updateWindow(self):
        self.resize()
        self._pep.setMinimumSize(
            SequenceIonsWidget.WIDTH, SequenceIonsWidget.HEIGHT)
        self.setFixedHeight(SequenceIonsWidget.HEIGHT)
        self.update()

    def clear(self):
        self._pep.sequence = ""
        self._pep.suffix = {}
        self._pep.prefix = {}
        self.update()


class observed_peptide(QWidget):
    """
    Used for creates a peptide sequence with its given ions.
    The ions can be stacked above each other, e.g. in case for
    a1, b1. Each amino letter is also separated by a line
    and prefixes are colored blue, otherwise suffixes are colored
    red.

    """

    def __init__(self):
        QWidget.__init__(self)
        self.initUI()

    def initUI(self):
        self.sequence = ""
        self.suffix = {}
        self.prefix = {}
        self.colors = {
            "black": QColor(0, 0, 0),
            "red": QColor(255, 0, 0),
            "blue": QColor(0, 0, 255),
        }

    def setSequence(self, seq):
        self.sequence = seq

    def setSuffix(self, lst):
        self.suffix = lst

    def setPrefix(self, lst):
        self.prefix = lst

    def paintEvent(self, event):
        qp = QPainter()
        qp.begin(self)
        qp.setRenderHint(QPainter.Antialiasing)
        qp.fillRect(event.rect(), QBrush(Qt.white))  # or changed to Qt.white
        self._drawPeptide(qp)
        qp.end()

    def _drawPeptide(self, qp):
        qp.setWindow(0, 0, SequenceIonsWidget.WIDTH, SequenceIonsWidget.HEIGHT)
        qp.setPen(self.colors["black"])
        qp.setFont(self.getFont_Pep())
        self._fragmentPeptide(qp)

    def _fragmentPeptide(self, qp):
        """
        In this function the sequence will be created stepwise.
        Each char of the sequence is drawn separately to add the
        lines between and the ions.

        1. Check if sequence is given, if so
        then transform seq into a dictionary
        (with the indices representing the positions of the chars).
        2. For each char in the sequence:
            First, calculate start position of char
            (be aware that the char rect is created
            at the left bottom corner of
            the char, meaning we have to add the height of the Font
            & possible suffixes to the starting height position
            to move it into the center of the window).

            Secound, calculate the center point for the vertical Line.
            The Line consists of a point start and point end.
            The starting line xPos yield in the
            start_point + blank - (SPACE/2),
            where blank represents the additional
            space from the starting point after each new char.

            Third, if prefix or suffix ions are given,
            then distinguish between suffix and prefix to draw the vertical
            line with either left or right line or both.

            Because of changing the fonts for the ions,
            the fonts needs to be reset.

        """
        SPACE = 8

        if self.sequence != "":

            seq = list(self.sequence)
            dict_seq = {i: seq[i] for i in range(0, len(seq))}

            metrics = QFontMetricsF(self.getFont_Pep())

            blank = 0
            for i, s in dict_seq.items():
                i_rev = self._getReverseIndex(i, dict_seq)

                width = metrics.boundingRect(s).width()
                height = metrics.boundingRect(s).height()

                start_point = 0

                # position of char with center indent
                position = QPointF(
                    start_point + blank,
                    SequenceIonsWidget.SUFFIX_HEIGHT + height
                )
                qp.drawText(position, s)

                # position lines for possible ions
                centerOfLine = \
                    (
                        SequenceIonsWidget.SUFFIX_HEIGHT + height - height / 4
                    ) - 1

                start_linePos = QPointF(
                    start_point + blank - (SPACE / 2),
                    centerOfLine - height / 2 - 2.5
                )
                end_linePos = QPointF(
                    start_linePos.x(), centerOfLine + height / 2 + 2.5
                )

                qp.setFont(self.getFont_Ion())
                metrics_ion = QFontMetricsF(self.getFont_Ion())

                if i in self.prefix:
                    left_linePos = self._drawIonsLines(
                        qp, start_linePos, end_linePos, SPACE, "prefix"
                    )
                    self._drawPrefixIon(qp, i, metrics_ion, left_linePos)

                # for given line of existing prefix, expand with given suffix
                if i in self.prefix and i_rev in self.suffix:
                    right_linePos = self._drawIonsLines(
                        qp, start_linePos, end_linePos, SPACE, "suffix"
                    )
                    self._drawSuffixIon(
                        qp, i_rev, metrics_ion, end_linePos, right_linePos
                    )

                elif i_rev in self.suffix and i not in self.prefix:
                    right_linePos = self._drawIonsLines(
                        qp, start_linePos, end_linePos, SPACE, "suffix"
                    )
                    self._drawSuffixIon(
                        qp, i_rev, metrics_ion, start_linePos, right_linePos
                    )

                blank += width + SPACE
                qp.setPen(self._getPen(self.colors["black"]))
                qp.setFont(self.getFont_Pep())

    def _drawPrefixIon(self, qp, index, metrics_ion, pos_left):
        qp.setPen(self._getPen(self.colors["blue"]))
        prefix_ions = sorted(self.prefix[index])
        blank_ion = 10

        for ion in prefix_ions:
            height_ion = metrics_ion.boundingRect(ion).height()
            pos_ion = QPointF(pos_left.x(), pos_left.y() + blank_ion)
            qp.drawText(pos_ion, ion)
            blank_ion += height_ion

    def _drawSuffixIon(
            self,
            qp,
            index_reverse,
            metrics_ion,
            pos_end,
            pos_right):
        qp.setPen(self._getPen(self.colors["red"]))
        suffix_ions = sorted(self.suffix[index_reverse], reverse=True)
        blank_ion = 5

        for ion in suffix_ions:
            height_ion = metrics_ion.boundingRect(ion).height()
            pos_ion = QPointF(pos_end.x() + 2.5, pos_right.y() - blank_ion)
            qp.drawText(pos_ion, ion)
            blank_ion += height_ion

    def _drawIonsLines(self, qp, pos_start, pos_end, SPACE, order):
        qp.setPen(self._getPen(self.colors["black"]))

        if order == "prefix":
            qp.drawLine(pos_start, pos_end)
            pos_left = QPointF(pos_end.x() - 2 * SPACE, pos_end.y())
            qp.drawLine(pos_end, pos_left)
            return pos_left

        if order == "suffix":
            qp.drawLine(pos_start, pos_end)
            pos_right = QPointF(pos_start.x() + 2 * SPACE, pos_start.y())
            qp.drawLine(pos_start, pos_right)
            return pos_right

    def getFont_Pep(self):
        font = QFont("Courier")
        font.setStyleHint(QFont.TypeWriter)
        font.setPixelSize(30)
        return font

    def getFont_Ion(self):
        font = QFont("Courier")
        font.setStyleHint(QFont.TypeWriter)
        font.setPixelSize(10)
        return font

    def _getPen(self, color):
        # style settings for the lines
        pen = QPen(color, 0.75, Qt.SolidLine)
        pen.setStyle(Qt.DashDotLine)
        return pen

    def _getReverseIndex(self, i, dict_seq):
        i_rev = 0
        if i != 0:
            i_rev = list(dict_seq.keys())[-i]
        return i_rev



from PyQt5.QtCore import (
    Qt,
    QAbstractTableModel,
    pyqtSignal,
    QItemSelectionModel,
    QSortFilterProxyModel,
    QSignalMapper,
    QPoint,
    QRegExp,
    QModelIndex,
)
from PyQt5.QtGui import QPen, QPainter
from PyQt5.QtWidgets import (
    QVBoxLayout,
    QWidget,
    QAction,
    QTableView,
    QMenu,
    QAbstractItemView,
    QItemDelegate,
)


class RTUnitDelegate(QItemDelegate):
    """
    Displays the minute values of the RT besides the RT values
    given in seconds. Through the delegate the RT values in
    the table are not changed, only difference is the display
    of the data in table_view. s
    """

    def __init__(self, parent, *args):
        super(RTUnitDelegate, self).__init__(parent, *args)

    def paint(self, painter, option, index):
        painter.save()
        painter.setPen(QPen(Qt.black))
        if index.isValid():
            rt_min = str(round(index.siblingAtColumn(2).data() * 1.0 / 60, 3))
            rt_sec = str(round(index.siblingAtColumn(2).data(), 3))
            text = "  " + rt_sec + "\t [" + rt_min + " Min" + "]"

            painter.setRenderHint(QPainter.Antialiasing)
            # adjust text into cell
            cell = option.rect
            cell.adjust(0, 5, 0, 5)
            painter.drawText(cell, Qt.AlignLeft, text)

            painter.restore()


class ScanTableWidget(QWidget):
    """
    Used for displaying information in a table.

    ===============================  =========================================
    **Signals:**
    sigScanClicked                   Emitted when the user has clicked
                                     on a row of the table and returns the
                                     current index. This index contains
                                     information about the current rows column
                                     data.

    ===============================  =========================================
    """

    sigScanClicked = pyqtSignal(QModelIndex, name="scanClicked")

    header = [
        "MS level",
        "Index",
        "RT (min)",
        "precursor m/z",
        "charge",
        "ID",
        "PeptideSeq",
        "PeptideIons",
    ]

    def __init__(self, ms_experiment, *args):
        QWidget.__init__(self, *args)
        self.ms_experiment = ms_experiment

        self.table_model = ScanTableModel(
            self, self.ms_experiment, self.header)
        self.table_view = QTableView()

        # register a proxy class for filering and sorting the scan table
        self.proxy = QSortFilterProxyModel(self)
        self.proxy.setSourceModel(self.table_model)

        self.table_view.sortByColumn(1, Qt.AscendingOrder)

        # setup selection model
        self.table_view.setSelectionMode(QAbstractItemView.SingleSelection)
        self.table_view.setSelectionBehavior(QAbstractItemView.SelectRows)
        self.table_view.setModel(self.proxy)
        self.table_view.setSelectionModel(QItemSelectionModel(self.proxy))

        # header
        self.horizontalHeader = self.table_view.horizontalHeader()
        self.horizontalHeader.sectionClicked.connect(self.onHeaderClicked)

        # enable sorting
        self.table_view.setSortingEnabled(True)

        # connect signals to slots
        self.table_view.selectionModel().currentChanged.connect(
            self.onCurrentChanged
        )  # keyboard moves to new row
        self.horizontalHeader.sectionClicked.connect(self.onHeaderClicked)

        layout = QVBoxLayout(self)
        layout.addWidget(self.table_view)
        self.setLayout(layout)

        # hide column 7 with the PepIon data, intern information usage
        self.table_view.setColumnHidden(7, True)

        # Add rt in minutes for better TIC interaction
        self.table_view.setItemDelegateForColumn(2, RTUnitDelegate(self))
        self.table_view.setColumnWidth(2, 160)

        # default : first row selected. in OpenMSWidgets

    def onRowSelected(self, index):
        """se_comment: hard-refactoring to comply to pep8"""
        if index.siblingAtColumn(1).data() is None:
            return  # prevents crash if row gets filtered out
        self.curr_spec = self.ms_experiment.getSpectrum(
            index.siblingAtColumn(1).data())
        self.scanClicked.emit(index)

    def onCurrentChanged(self, new_index, old_index):
        self.onRowSelected(new_index)

    def onHeaderClicked(self, logicalIndex):
        if logicalIndex != 0:
            return  # allow filter on first column only for now

        self.logicalIndex = logicalIndex
        self.menuValues = QMenu(self)
        self.signalMapper = QSignalMapper(self)

        # get unique values from (unfiltered) model
        valuesUnique = set(
            [
                self.table_model.index(row, self.logicalIndex).data()
                for row in range(
                    self.table_model.rowCount(
                        self.table_model.index(-1, self.logicalIndex)
                    )
                )
            ]
        )

        if len(valuesUnique) == 1:
            return  # no need to select anything

        actionAll = QAction("Show All", self)
        actionAll.triggered.connect(self.onShowAllRows)
        self.menuValues.addAction(actionAll)
        self.menuValues.addSeparator()

        """se_comment: hard-refactoring to comply to pep8"""
        l: enumerate = enumerate(sorted(list(set(valuesUnique))))
        for actionNumber, actionName in l:
            action = QAction(actionName, self)
            self.signalMapper.setMapping(action, actionNumber)
            action.triggered.connect(self.signalMapper.map)
            self.menuValues.addAction(action)

        self.signalMapper.mapped.connect(self.onSignalMapper)

        # get screen position of table header and open menu
        headerPos = self.table_view.mapToGlobal(self.horizontalHeader.pos())
        posY = headerPos.y() + self.horizontalHeader.height()
        posX = headerPos.x() + \
            self.horizontalHeader.sectionPosition(self.logicalIndex)
        self.menuValues.exec_(QPoint(posX, posY))

    def onShowAllRows(self):
        filterColumn = self.logicalIndex
        filterString = QRegExp("", Qt.CaseInsensitive, QRegExp.RegExp)

        self.proxy.setFilterRegExp(filterString)
        self.proxy.setFilterKeyColumn(filterColumn)

    def onSignalMapper(self, i):
        stringAction = self.signalMapper.mapping(i).text()
        filterColumn = self.logicalIndex
        filterString = QRegExp(
            stringAction, Qt.CaseSensitive, QRegExp.FixedString)

        self.proxy.setFilterRegExp(filterString)
        self.proxy.setFilterKeyColumn(filterColumn)


class ScanTableModel(QAbstractTableModel):
    """
        TODO: directly read model data from MSExperiment to remove copies
    """

    def __init__(self, parent, ms_experiment, header, *args):
        QAbstractTableModel.__init__(self, parent, *args)
        self.header = header

        # create array with MSSpectrum
        self.scanRows = self.getScanListAsArray(
            ms_experiment)  # data type: list

    def getScanListAsArray(self, ms_experiment):
        scanArr = []
        for index, spec in enumerate(ms_experiment):
            MSlevel = "MS" + str(spec.getMSLevel())
            RT = spec.getRT()
            prec_mz = "-"
            charge = "-"
            native_id = spec.getNativeID()
            if len(spec.getPrecursors()) == 1:
                prec_mz = spec.getPrecursors()[0].getMZ()
                charge = spec.getPrecursors()[0].getCharge()
            PeptideSeq = "-"
            PeptideIons = "-"

            scanArr.append(
                [
                    MSlevel,
                    index,
                    RT,
                    prec_mz,
                    charge,
                    native_id,
                    PeptideSeq,
                    PeptideIons,
                ]
            )
        return scanArr

    def headerData(self, col, orientation, role):
        if orientation == Qt.Horizontal and role == Qt.DisplayRole:
            return self.header[col]
        return None

    def rowCount(self, parent):
        return len(self.scanRows)

    def columnCount(self, parent):
        return len(self.header)

    def setData(self, index, value, role):
        if index.isValid() and role == Qt.DisplayRole:
            self.scanRows[index.row()][index.column()] = value
            self.dataChanged.emit(index, index, {Qt.DisplayRole, Qt.EditRole})
            return value
        return None

    def flags(self, index):
        if not index.isValid():
            return None
        return Qt.ItemIsEnabled | Qt.ItemIsSelectable

    def data(self, index, role):
        if not index.isValid():
            return None
        value = self.scanRows[index.row()][index.column()]
        if role == Qt.EditRole:
            return value
        elif role == Qt.DisplayRole:
            return value



import setuptools

with open("pyopenms-extra/README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="pyopenms-extra-Fabian-Yannik-Laura", # Replace with your own username
    version="1.0",
    author="Fabian Rosner",
    author_email="fabian@rosner.email",
    description="Teamprojekt SS20 Yannik Haller Fabian Rosner Laura Schöneberg",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://fabian1567.github.io/Teamprojekt-SS20-Laura-Yannik-Fabian/",
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.6',
)


import sys
from GUI_EXAMPLE_BASE import GUI_EXAMPLE_BASE
from PyQt5.QtWidgets import QApplication
sys.path.insert(0, "../view")
from SequenceIonsWidget import SequenceIonsWidget

if __name__ == "__main__":
    app = QApplication(sys.argv)
    ex = GUI_EXAMPLE_BASE()  # plain QMainWindow with basic layout and menu bar
    example_widget = SequenceIonsWidget(ex)
    example_widget.setPeptide("HLDKIDMLKSLD")
    example_widget.setPrefix(
        {1: ["a1", "b1", "c1"], 3: ["a3", "b3", "c3"], 5: ["a3", "b3", "c3"]}
    )
    example_widget.setSuffix(
        {1: ["x1", "x3"], 2: ["x2", "y2"], 4: ["x4", "y4", "z4"]})
    ex.setExampleWidget(example_widget)
    ex.show()
    sys.exit(app.exec_())



import sys
import webbrowser
from PyQt5 import QtGui, QtWidgets
from PyQt5.QtWidgets import QApplication, QWidget, QTableWidget, QVBoxLayout, QTableWidgetItem, QPushButton, QFileDialog
from mzTabTableWidget import mzTabTableWidget as mz

"""
Helperwidget for using/debugging the mzTabTableWidget on its own
"""

class Window(QWidget):
    def __init__(self):
        super().__init__()

        self.title = "mzTabTableWidget"
        self.top = 100
        self.left = 100
        self.width = 500
        self.height = 500

        self.vBox = QVBoxLayout()

        self.InitWindow()

    def InitWindow(self):
        #initializes simple window containing the mzTabTableWidget and a loadbutton
        self.setWindowIcon(QtGui.QIcon("icon.png"))
        self.setWindowTitle(self.title)
        self.setGeometry(self.top, self.left, self.width, self.height)

        self.mzTabTableWidget = mz()

        self.loadButton = QtWidgets.QPushButton(self)
        self.loadButton.setText("load")
        self.loadButton.clicked.connect(self.loadFile)

        self.vBox.addWidget(self.loadButton)
        self.vBox.addWidget(self.mzTabTableWidget)

        self.setLayout(self.vBox)
        self.show()

    def loadFile(self):
        #opens filedialog and loads chosen mzTab in the tableWidget
        self.filename = QFileDialog.getOpenFileName()
        self.mzTabTableWidget.readFile(self.filename[0])

# the following is here to test the widget on its own, leave it commented if you want to use it elsewhere
"""
if __name__== '__main__':
    app = QApplication(sys.argv)
    ex = mzTabTableWidget()
    sys.exit(app.exec_())
"""


import numpy as np
import pyqtgraph as pg
from PyQt5.QtCore import QPointF
from pyqtgraph import PlotWidget

pg.setConfigOption("background", "w")  # white background
pg.setConfigOption("foreground", "k")  # black peaks


class ErrorWidget(PlotWidget):
    """
    Used to plot a error plot to display the derivation
        between exact mass and theoretical mass.
    """

    def __init__(self, *args):
        PlotWidget.__init__(self)

        self.setLimits(xMin=0)
        self.setLabel("bottom", "m/z")
        self.setLabel("left", "ppm")
        self._mzs = np.array([])
        self._ppm = np.array([])
        self._color_lib = np.array([])
        self.getViewBox().sigXRangeChanged.connect(self._autoscaleYAxis)
        self.setMouseEnabled(x=True, y=False)

    def setMassErrors(self, mz, ppm, colors):
        """
        Used for creating new error plot
            with the m/z of the peptides fragments.
        :param mz: An numpy array of m/z
            (mass divided by charge number) of the ions
            (starting with xyz or abc)
        :param ppm: An numpy array of random numbers,
            ppm needs to be calculated
        :param colors: An numpy array of colors consisting of red and blue
            (representing prefix -> blue and suffix -> red ions)

        """
        self._mzs = mz
        self._ppm = ppm
        self._color_lib = colors
        self.redraw()

    def redraw(self):
        self.plot(clear=True)
        self.setXRange(np.amin(self._mzs), np.amax(self._mzs))
        self._autoscaleYAxis()
        self._plotHorizontalLine()
        self._plotMassErrors()

    def _plotMassErrors(self):
        scattergraph = pg.ScatterPlotItem()
        points = []
        for i in range(0, self._ppm.size):
            points.append(
                {
                    "pos": (self._mzs[i], self._ppm[i]),
                    "brush": pg.mkBrush(self._color_lib[i]),
                }
            )
            scattergraph.addPoints(points)
            self.addItem(scattergraph)

    def _plotHorizontalLine(self):
        horizontalLine = pg.InfiniteLine(
            pos=QPointF(0.0, 0.0), angle=0, pen=pg.mkColor("k")
        )
        self.addItem(horizontalLine)

    def _autoscaleYAxis(self):
        x_range = self.getAxis("bottom").range
        if x_range == [0, 1]:  # workaround for axis sometimes not being set
            x_range = [np.amin(self._mzs), np.amax(self._mzs)]
        self.currMaxY = self._getMaxMassErrorInRange(x_range)
        if self.currMaxY:
            self.setYRange(self.currMaxY * (-1), self.currMaxY, update=False)

    def _getMaxMassErrorInRange(self, xrange):
        left = np.searchsorted(self._mzs, xrange[0], side="left")
        right = np.searchsorted(self._mzs, xrange[1], side="right")
        return np.amax(abs(self._ppm[left:right]), initial=1)



import os
import sys
from PyQt5.QtWidgets import QMainWindow, QApplication, QPushButton, QWidget, QAction, QTabWidget, QVBoxLayout, \
    QMessageBox, QFileDialog
from PyQt5.QtGui import QIcon
from PyQt5.QtCore import pyqtSlot
import glob
sys.path.append('../apps')
sys.path.append('../model')
sys.path.append('../controller')

from filehandler import FileHandler
from tableDataFrame import TableDataFrame

from mzTabTableWidget import mzTabTableWidget
from GUI_FastaViewer import GUI_FastaViewer
from ErrorWidget import ErrorWidget
from HomeTabWidget import HomeTabWidget

from SpecViewer import App
from ConfigView import ConfigView
from mzMLTableView import mzMLTableView


class AppGUITabs(QMainWindow):

    def __init__(self):
        super().__init__()
        self.title = 'GUI_Tabs'
        self.left = 0
        self.top = 0
        self.width = 300
        self.height = 200
        self.setWindowTitle(self.title)
        self.setGeometry(self.left, self.top, self.width, self.height)
        
        self.table_widget = MyTableWidget(self)
        self.setCentralWidget(self.table_widget)
        
        self.show()


class MyTableWidget(QWidget):
    """
    Shows different widgets to evaluate experiment data, lets user load data and run a proteomicsLFQ command on given
    data. Files can be chosen manually or loaded automatically from a chosen project folder.
    ...
    Attributes
    ----------
    loadedFolder : String
        path to project folder
    loadedFasta : String
        path to loaded .fasta file
    loadedIni : String
        path to loaded .ini file
    loadedTsv : String
        path to loaded .tsv file
    initButton : QPushButton
        button to automatically load files        
    loadButton : QPushButton
        button to run proteomicsLFQ command
    self.tabs :
        main widget containing all the tabs
    self.tab1 :
        first tab containing the HomeTabWidget
    self.tab2 :
        second tab containing the ConfigView widget
    self.tab3 :
        third tab containing the mzMLTableView widget
    self.tab4 :
        fourth tab containing the FastaViewer widget
    self.tab5 :
        fifth tab containing the mzTabTableWidget
    self.tab6 :
        sixth tab containing the SpecViewer widget
    Methods
    -------
    _init_(self)
        Sets Window size, initializes variables
    show_popup(self)
        asks user to pick a folder from which files should be loaded automatically
    popupbutton_clicked(self, i)
        lets user pick a folder and loads all data in given folder
    PopupFolder(self)
        asks user to pick a project folder
    loadFolder(self)
        lets user pick a project folder
    PopupFasta(self)
        asks user to load .fasta file if missing
    loadFasta(self)
        lets user pick and loads a .fasta file
    PopupTsv(self)
        asks user to load .tsv file if missing
    loadTsv(self)
        lets user pick and loads a .tsv file
    PopupIni(self)
        asks user to load .ini file if missing
    loadIni(self, i)
        lets user pick or automatically creates .ini file
    LFQ(self)
        asks the user to load missing files and runs LFQ command
    """
    def __init__(self, parent):
        super(QWidget, self).__init__(parent)
        self.layout = QVBoxLayout(self)
        
        self.loadedFolder = ""
        self.loadedFasta = ""
        self.loadedIni = ""
        self.loadedTsv = ""
        
        # add button to automatically load data into widgets
        self.initButton = QPushButton(self)  
        self.initButton.setText("Load Data")
        self.initButton.setFixedSize(220,25)
        self.initButton.clicked.connect(self.show_popup)  

        # add button to run proteimicsLFQ command
        self.loadButton = QPushButton(self)  
        self.loadButton.setText("Run ProteomicsLFQ")
        self.loadButton.setFixedSize(220,25)
        self.loadButton.clicked.connect(self.LFQ)

        # initialize tab screen
        self.tabs = QTabWidget()
        self.tab1 = HomeTabWidget()
        self.tab2 = ConfigView()
        self.tab3 = mzMLTableView()
        self.tab4 = mzTabTableWidget()
        self.tab5 = GUI_FastaViewer()
        self.tab6 = App()

        self.tabs.resize(300, 200)

        # add tabs
        self.tabs.addTab(self.tab1, "Home")
        self.tabs.addTab(self.tab2, "Ini-Config")
        self.tabs.addTab(self.tab3, "Experimental Design")
        self.tabs.addTab(self.tab4, "PSM/PRT Table")
        self.tabs.addTab(self.tab5, "Fasta-Viewer")
        self.tabs.addTab(self.tab6, "Spectrum Viewer")

        # add tabs to widget
        self.layout.addWidget(self.initButton)
        self.layout.addWidget(self.loadButton)
        self.layout.addWidget(self.tabs)
        self.setLayout(self.layout)

    def show_popup(self):
        '''Shows a popup asking you if you want to load all data automatically by choosing a folder
            containing all the needed files.
        -------
        nothing
        '''
        msg = QMessageBox()
        msg.setWindowTitle("Attention!")
        msg.setText("Want data to be loaded automatically?  ")
        msg.setIcon(QMessageBox.Question)
        msg.setDetailedText(
            "If you choose to load data automatically, please enter the folder that contains entire data (for different widgets). Otherwise, please enter data manually through widgets. Thank you.")
        msg.setStandardButtons(QMessageBox.Yes | QMessageBox.Cancel)
        msg.buttonClicked.connect(self.popupbutton_clicked)

        x = msg.exec_()

    def popupbutton_clicked(self, i):
        '''If you choose to load data automatically asks you to choose a folder which contains the needed data
            and then loads and shows fasta, tsv and ini. If no ini is found one is created.
        Parameters
        ----------
        i : Information about clicked Button
        Returns
        -------
        nothing
        '''
        if i.text() == "&Yes":
            self.AutoLoadedData = True #neu
            dialog = QFileDialog(self)
            self.loadedFolder = dialog.getExistingDirectory()
            print(self.loadedFolder)

            os.chdir(self.loadedFolder)
            for file in glob.glob("*.fasta"):
                self.tab5.loadFile(file)
                self.loadedFasta = file

            for file in glob.glob("*.ini"):
                self.tab2.openXML(file)
                self.loadedIni = file

            if self.loadedIni == "":
                os.system("ProteomicsLFQ -write_ini generated.ini")
                self.tab2.openXML("generated.ini")
                self.loadedIni = "generated.ini"

            for file in glob.glob("*.tsv"):
                self.loadedTsv = file
                self.tab3.loadDir(self.loadedFolder)


    def PopupFolder(self):
        '''Shows a popup asking you to choose a project folder, which must be the folder containing
            the mzML and idXML files.
        Returns
        -------
        nothing
        '''
        msg = QMessageBox()
        msg.setWindowTitle("Attention!")
        msg.setText("Please choose a Project Folder \n(must be the one containing the mzML and idXML files)")
        msg.setStandardButtons(QMessageBox.Ok)
        msg.buttonClicked.connect(self.loadFolder)

        x = msg.exec_()

    def loadFolder(self):
        '''Opens a file Dialog in which you can choose your project folder.
        Returns
        -------
        nothing
        '''
        dialog = QFileDialog(self)
        self.loadedFolder = dialog.getExistingDirectory()

    def PopupFasta(self):
        '''Shows a popup asking you to choose a .fasta file.
        Returns
        -------
        nothing
        '''
        msg = QMessageBox()
        msg.setWindowTitle("Attention!")
        msg.setText("Please choose a Fasta file")
        msg.setStandardButtons(QMessageBox.Ok)
        msg.buttonClicked.connect(self.loadFasta)

        x = msg.exec_()


    def loadFasta(self):
        '''Opens a file Dialog in which you can choose your .fasta file, when file is chosen it's shown in the
            Fasta-Viewer tab.
        Returns
        -------
        nothing
        '''
        fileDialog = QFileDialog.getOpenFileName(self, "Choose Fasta","","Fasta files (*.fasta)",
                                                 "Fasta files (*.fasta)")
        fileName = fileDialog[0]
        self.tab5.loadFile(fileName)
        self.loadedFasta = fileName

    def PopupTsv(self):
        '''Shows a popup asking you to choose a .tsv file.
        Returns
        -------
        nothing
        '''
        msg = QMessageBox()
        msg.setWindowTitle("Attention!")
        msg.setText("Please choose a .tsv")
        msg.setStandardButtons(QMessageBox.Ok)
        msg.buttonClicked.connect(self.loadTsv)

        x = msg.exec_()

    def loadTsv(self):
        '''Opens a file Dialog in which you can choose your .tsv file, when file is chosen the respective
            mzML and idXML files from the project folder are shown in the Experimental Design tab
        Returns
        -------
        nothing
        '''
        fileDialog = QFileDialog.getOpenFileName(self, "Choose .tsv","",".tsv files (*.tsv)",
                                                 ".tsv files (*.tsv)")
        fileName = fileDialog[0]
        self.loadedTsv = fileName
        self.tab3.loadDir(self.loadedFolder)

    def PopupIni(self):
        '''Shows a popup asking you to choose a .ini file.
        Returns
        -------
        nothing
        '''
        msg = QMessageBox()
        msg.setWindowTitle("Attention!")
        msg.setText("Do you want to choose a .ini ? \nIf you choose cancel a .ini will be automatically generated")
        msg.setStandardButtons(QMessageBox.Yes|QMessageBox.Cancel)
        msg.buttonClicked.connect(self.loadIni)

        x = msg.exec_()

    def loadIni(self, i):
        '''If you choose to load a .ini a file dialog opens and you can choose it and it will be shown in
            the Ini-Config tab. If you don't want to load a .ini one is created and shown automatically.
        Parameters
        ----------
        i : Information about clicked Button
        Returns
        -------
        nothing
        '''
        if i.text() == "&Yes":
            fileDialog = QFileDialog.getOpenFileName(self, "Choose .ini","",".ini files (*.ini)",
                                                 ".ini files (*.ini)")
            fileName = fileDialog[0]

            self.tab2.generateTreeWidgetItem(fileName)
            self.loadedIni = fileName

        else:
            os.chdir(self.loadedFolder)
            os.system("ProteomicsLFQ -write_ini generated.ini")
            self.tab2.openXML("generated.ini")
            self.loadedIni = "generated.ini"

    def LFQ(self):
        '''Checks if all needed files are loaded. If more files are needed popups are shown to guide the user
            through loading the additional files. Takes all the loaded files and runs the LFQ command and presents
            the created .mzTab file in the PSM/PRT Table tab.
        Returns
        -------
        nothing
        '''
        if self.loadedFolder == "":
            self.PopupFolder()

        if self.loadedFasta == "" and self.tab5.path == "":
            self.PopupFasta()

        elif self.tab5.path != "":
            self.loadedFasta = self.tab5.path

        if self.loadedTsv == "" and self.tab3.path == "":
            self.PopupTsv()

        elif self.tab3.path != "":
            self.loadedTsv = self.tab3.path

        if self.loadedIni == "" and self.tab2.path == "":
            self.PopupIni()

        elif self.tab2.path != "":
            self.loadedIni = self.tab2.path

        os.chdir(self.loadedFolder)
        mzML = sorted(glob.glob("*.mzML"))
        idXML = sorted(glob.glob("*.idXML"))

        self.tab2.saveTmpFile()

        if len(mzML) == len(idXML):

            command = "ProteomicsLFQ -in    "

            for file in mzML:
                command += file + " "

            command += "-ids "

            for file in idXML:
                command += file + " "

            command += "-design " + self.loadedTsv + " "
            command += "-fasta " + self.loadedFasta + " "
            command += "-ini tmp.ini " \
                       "-out_cxml BSA.consensusXML.tmp " \
                       "-out_msstats BSA.csv.tmp " \
                       "-out BSA.mzTab.tmp " \
                       "-threads " + self.tab1.loadedThreads + " " \
                       "-proteinFDR " + self.tab1.loadedFDR + " "

            print(command)
            os.chdir(self.loadedFolder)
            os.system(command)

            for file in glob.glob("*.mzTab.tmp"):
                self.tab4.readFile(file)

            os.remove("tmp.ini")


if __name__ == '__main__':
    app = QApplication(sys.argv)
    ex = AppGUITabs()
    sys.exit(app.exec_())
